from ralium.element import HTMLElement
from ralium.webpage import WebHook
from ralium.config import WindowConfig

import webview

class Window:
    def __init__(self, webhooks: list[WebHook], config: WindowConfig | None = None) -> None: ...
    
    def getElementById(self, id: str) -> HTMLElement | None: ...
    def load_handler(self, window: webview.Window) -> None: ...
    def shutdown(self) -> None: ...
    def display(self, url: str) -> None: ...
    def render(self, html: str) -> None: ...
    def start(self) -> None: ...
    def show(self) -> None: ...
    def hide(self) -> None: ...


from ralium.errors import DisableWarnings
import PyInstaller.__main__
import sys
import os

HTML_FILE_EXTENSIONS = [".htm", ".html"]

def _exe_str_error(name, var):
    return TypeError(f"Expected executable {name} to be of type 'str', instead got '{type(var)}'")

def _add_data_arg(src, dst):
    return f'--add-data={src}{os.pathsep}{dst}'

def collect_webfolder(webfolder):
    """
    Collects all necessary files from a Ralium Web Folder used within the executable main Python file.

    :param webfolder: The path to the project folder.
    
    :returns: A dictionary where the keys contain the absolute paths and the values represent the relative paths.
    """

    webfiles = {}
    webfolder = os.path.abspath(webfolder)
    webroutes = os.path.join(webfolder, "routes")
    cssfolder = os.path.join(webfolder, "styles")
    imgfolder = os.path.join(webfolder, "images")
    webfolder_name = os.path.basename(webfolder)
    
    if os.path.exists(webroutes):
        for root, _, files in os.walk(webroutes):
            dest = os.path.normpath(f"{webfolder_name}\\{root.removeprefix(webfolder)}")
            webfiles.update({os.path.join(root, file): os.path.join(dest, file) for file in files if os.path.splitext(file)[-1] in HTML_FILE_EXTENSIONS})
    
    if os.path.exists(cssfolder):
        for root, _, files in os.walk(cssfolder):
            dest = os.path.normpath(f"{webfolder_name}\\{root.removeprefix(webfolder)}")
            webfiles.update({os.path.join(root, file): os.path.join(dest, file) for file in files if os.path.splitext(file)[-1] == ".css"})
    
    if os.path.exists(imgfolder):
        for root, _, files in os.walk(imgfolder):
            dest = os.path.normpath(f"{webfolder_name}\\{root.removeprefix(webfolder)}")
            webfiles.update({os.path.join(root, file): os.path.join(dest, file) for file in files})
    
    return webfiles

def setup(
    pyfile,
    name = None,
    icon = None,
    onefile = True,
    warnings = False,
    noconsole = True,
    webfolder = None,
    *pyi_args
):
    """
    Compiles a Ralium project using PyInstaller.

    :param pyfile: Python file to compile.
    :param name: Display name for the executable.
    :param icon: Display icon for the executable.
    :param onefile: Bundles the executable as a standalone file.
    :param warnings: Calls the `DisableWarnings` to prevent warnings.
    :param noconsole: Prevents a console from being displayed.
    :param webfolder: Directory of the project.
    :param pyi_args: Extra parameters for PyInstaller to use.

    :raises TypeError: If the name or icon is not a `str` or `None`.
    :raises RuntimeError: If this function is called within an already compiled executable file.
    :raises FileNotFoundError: If a certain file path doesn't exist.
    """

    if getattr(sys, "frozen", False):
        raise RuntimeError("Ralium setup cannot be ran from an executable file.")
    
    if not os.path.exists(pyfile):
        raise FileNotFoundError(f"Failed to find python file '{pyfile}'")

    args = [pyfile, *pyi_args]

    if name is not None:
        if not isinstance(name, str):
            raise _exe_str_error("name", name)
        
        args.append(f"--name={name}")
    
    if icon is not None:
        if not isinstance(icon, str):
            raise _exe_str_error("icon", icon)

        if not os.path.exists(icon):
            raise FileNotFoundError(f"Failed to find icon file with path: '{icon}'")
        
        args.append(f"--icon={icon}")
    
    if onefile:
        args.append(f"--onefile")
    
    if not warnings:
        DisableWarnings()
    
    if noconsole:
        args.append(f"--noconsole")
    
    if webfolder:
        for src, dst in collect_webfolder(webfolder).items():
            args.append(_add_data_arg(src, dst))

    PyInstaller.__main__.run(args)


from ralium._util import (
    FunctionType,
    ModuleType,
    Callable,
    Self,
    Any, 
    _RT
)

from ralium.window import Window

def _process_api_calldata(window: Window, calldata: list[Any]) -> list[Any]: ...

class WebApi:
    def __init__(self) -> None: ...
    def bridge(self, func: FunctionType, calldata: list[Any] | None = None) -> Any: ...

class WebBridge:
    def __init__(self, *functions: tuple[FunctionType]) -> None: ...
    def __str__(self) -> str: ...
    def new(self, function: FunctionType) -> None: ...
    def namespace(self, 
        alias: str, 
        *functions: tuple[FunctionType], 
        **named_functions: dict[str, FunctionType]
    ) -> None: ...
    def _function_template(self, name: str, string: str) -> str: ...
    
class WebApiNamespaceDict(dict):
    def add_namespace(self, namespace: ModuleType | dict[str, FunctionType]) -> None: ...

class WebApiFunctionDict(dict):
    def add_function(self, function: FunctionType) -> None: ...
    
class WebEngine:
    def __init__(self, window: Window) -> None: ...
    def __str__(self) -> str: ...

    @property
    def running(self) -> bool: ...
    def _init_server(self) -> tuple[int, Callable[..., _RT]]: ...
    def close(self) -> None: ...
    def create_api(engine: Self) -> WebApi: ...
    def create_event(self) -> Callable[..., _RT]: ...


import http.server

import platform
import ctypes
import winreg
import sys
import os

__all__ = ["BasicHTTPServer", "_norm_url", "_get_path_limit", "_get_path", "_is_markup_filelike"]

class BasicHTTPServer(http.server.SimpleHTTPRequestHandler):
    pass

def _norm_url(path):
    return os.path.normpath(f"/{path.lstrip('/\\')}").replace("\\", "/")

def _get_path_limit_winreg():
    with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r"SYSTEM\CurrentControlSet\Control\FileSystem") as key:
        max_path_length, _ = winreg.QueryValueEx(key, "LongPathsEnabled")
        return max_path_length

def _get_path_limit():
    system = platform.system()

    try:
        if system == 'Windows':
            return os.path.getconf('PC_NAME_MAX')
        
        if system == 'Darwin':
            return os.pathconf('/', 'PC_NAME_MAX')

        libc = ctypes.CDLL('libc.so.6')
        return libc.fpathconf('/', 261)
    except:
        if system == "Windows":
            return 32767 if _get_path_limit_winreg() else 255

def _get_path(path):
    if getattr(sys, "frozen", False):
        return os.path.abspath(os.path.join(sys._MEIPASS, path))
    return os.path.abspath(path)

# A copy of the 'BeautifulSoup._markup_resembles_filename' function
def _is_markup_filelike(markup):
    path_characters = '/\\'
    extensions = ['.html', '.htm', '.xml', '.xhtml', '.txt']
    if isinstance(markup, bytes):
        path_characters = path_characters.encode("utf8")
        extensions = [x.encode('utf8') for x in extensions]
    filelike = False
    if any(x in markup for x in path_characters):
        filelike = True
    else:
        lower = markup.lower()
        if any(lower.endswith(ext) for ext in extensions):
            filelike = True
    return filelike